@@ -48,13 +48,13 @@ uiFun <- function(beta, Y, X, delta, clsize, sigma, n, Z, weight, smooth = TRUE,
   ans <- ans - constant
 }
 
-uilogFun <- function(beta, Y, X, delta, clsize, sigma, n, Z, weight, smooth = TRUE, constant = 0, rank = "logrank") {
+uilogFun <- function(beta, Y, X, delta, clsize, sigma, n, Z, weight, smooth = TRUE, constant = 0, rankweight = "logrank") {
   N <- sum(clsize)
   p <- ncol(X)
   sn <- vector("double", p)
   ans <- numeric(p)
   pw <- rep(1, N)
-  if (rank == "PW" | rank == "Prentice-Wilcoxon" | rank == "GP") {
+  if (rankweight == "PW" | rankweight == "Prentice-Wilcoxon" | rankweight == "GP") {
     en <- Y - X %*% beta
     ik <- rep(1:N, each=N)
     jl <- rep(1:N, N)
@@ -70,7 +70,7 @@ uilogFun <- function(beta, Y, X, delta, clsize, sigma, n, Z, weight, smooth = TR
     Shat[dummy[ord]] <- Shati
     pw <- Shat
   }
-  if (rank == "GP") {
+  if (rankweight == "GP") {
       pw <- pw ^ p
   }
   if (smooth == TRUE) {
@@ -82,14 +82,14 @@ uilogFun <- function(beta, Y, X, delta, clsize, sigma, n, Z, weight, smooth = TR
   ans - constant
 }
 
-viEmp <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, mb = TRUE, zbeta = FALSE, smooth = TRUE, rank = "gehan"){
+viEmp <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), nres = 500, mb = TRUE, zbeta = FALSE, smooth = TRUE, rankweight = "gehan"){
   p <- ncol(X)
   clsize <- unlist(lapply(split(id, id), length))
   n <- length(clsize)
   N <- sum(clsize)
   sigma <- diag(p)
-  UnV <- zmat <- matrix(0, ncol = M, nrow = p)
-  for (i in 1:M) {
+  UnV <- zmat <- matrix(0, ncol = nres, nrow = p)
+  for (i in 1:nres) {
     if ( mb == TRUE) {
       Z <- rep(rexp(length(clsize)), clsize)
     }
@@ -106,20 +106,20 @@ viEmp <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, mb =
     }
     sn <- vector("double", p)
     if (smooth == TRUE) {
-        if (rank == "gehan") {
+        if (rankweight == "gehan") {
             UnV[,i] <- as.vector(.C("ufun", as.double(newbeta), as.double(Y), as.double(X), as.double(delta), as.integer(clsize), as.double(sigma), as.integer(n), as.integer(p), as.integer(N), as.double(Z), as.double(weight), out = as.double(sn), PACKAGE = "aftgee")$out) / n
         }
-        if (rank != "gehan") {
-            UnV[,i] <- uilogFun(newbeta, Y, X, delta, clsize, sigma, n, Z, weight, smooth, constant = 0, rank) / n
+        if (rankweight != "gehan") {
+            UnV[,i] <- uilogFun(newbeta, Y, X, delta, clsize, sigma, n, Z, weight, smooth, constant = 0, rankweight) / n
             ## as.vector(.C("ulogfun", as.double(newbeta), as.double(Y), as.double(X), as.double(delta), as.integer(clsize), as.double(sigma), as.integer(n), as.integer(p), as.integer(N), as.double(Z), as.double(weight), out = as.double(sn), PACKAGE = "aftgee")$out) / n
         }
     }
     if (smooth == FALSE) {
-        if (rank == "gehan") {
+        if (rankweight == "gehan") {
             UnV[,i] <- as.vector(.C("unsfun", as.double(newbeta), as.double(Y), as.double(X), as.double(delta), as.integer(clsize), as.double(sigma), as.integer(n), as.integer(p), as.integer(N), as.double(Z), as.double(weight), out = as.double(sn), PACKAGE = "aftgee")$out) / n
         }
-        if (rank != "gehan") {
-            UnV[,i] <- uilogFun(newbeta, Y, X, delta, clsize, sigma, n, Z, weight, smooth, constant = 0, rank) / n
+        if (rankweight != "gehan") {
+            UnV[,i] <- uilogFun(newbeta, Y, X, delta, clsize, sigma, n, Z, weight, smooth, constant = 0, rankweight) / n
             ## as.vector(.C("ulognsfun", as.double(newbeta), as.double(Y), as.double(X), as.double(delta), as.integer(clsize), as.double(sigma), as.integer(n), as.integer(p), as.integer(N), as.double(Z), as.double(weight), out = as.double(sn), PACKAGE = "aftgee")$out) / n
         }
     }
@@ -128,7 +128,7 @@ viEmp <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, mb =
   list(vi = vi, zmat = zmat, UnV = UnV)
 }
 
-get_si <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, pn = 1, rank = "gehan") {
+get_si <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), nres = 500, rankweight = "gehan", given = TRUE) {
     p <- ncol(X)
     clsize <- unlist(lapply(split(id, id), length))
     N <- sum(clsize)
@@ -151,7 +151,7 @@ get_si <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, pn
     ind <- ifelse(edif <= 0, 1, 0)
     minEn <- ifelse(edif <= 0, ik, jl)
     si <- s <- NULL
-    if (rank == "gehan") {
+    if (rankweight == "gehan") {
         s <- weight[ik] * weight[jl] * delta[ik] * ind * xdif + weight[ik] * weight[jl] * log(Shat[minEn]) * xdif ## need second weight[ik]?
         s <- ifelse(s == Inf, 0, s)
         s <- ifelse(s == -Inf, 0, s)
@@ -159,7 +159,7 @@ get_si <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, pn
         ##  s <- s / N
         s <- rowsum(s, ik)  ## N X p
     }
-    if (rank == "logrank") {
+    if (rankweight == "logrank") {
         haz <- -1 * log(Shat)
         haz <- ifelse(haz == Inf, 0, haz)
         haz <- ifelse(haz == -Inf, 0, haz)
@@ -174,57 +174,81 @@ get_si <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, pn
         s2[dummy[ord],] <- s2s
         s <- delta[1:N] * si1 - s2
     }
-    s
+    s <- subset(s, given)
 }
 
-viClo <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, pn = 1, rank = "gehan") {
-    p <- ncol(X)
-    clsize <- unlist(lapply(split(id, id), length))
-    n <- length(clsize)
-    N <- sum(clsize)
-    s <- get_si(beta, Y, delta, X, id, weight, M, pn, rank)
+get_vi <- function(s, id, delta, weight, n, N, pn) {
     s1 <- rowsum(s, group = id)
     si1 <- lapply(split(s1, 1:n), function(x) x %o% x)
     s11 <- mapply("*", si1, weight, SIMPLIFY = FALSE)
     vi <- Reduce("+", s11) / N
+    p <- ncol(s)
+    v2 <- matrix(0, ncol = p, nrow = p)
     if (pn <1) {
         s2 <- rowsum(s * (1 - delta), group = id)
         si2 <- lapply(split(s2, 1:n), function(x) x %o% x)
-        ## s22 <- mapply("*", si2, weight * (1 - delta), SIMPLIFY = FALSE)
         s22 <- mapply("*", si2, weight, SIMPLIFY = FALSE)
         v21n <- Reduce("+", s22) / N
         s3 <- s * weight * (1 - delta)
         v22n <- apply(s3, 2, sum) / N
         v22n <- v22n %o% v22n
-        vi <- vi+ ( (1 - pn) / pn) * (v21n- v22n)
+        v2 <- ( (1 - pn) / pn) * (v21n - v22n)
+        ## vi <- vi+ ( (1 - pn) / pn) * (v21n - v22n)
     }
-    vi <- vi / N
-    list(vi = vi, s  = s)
+    vi <- vi / n
+    v2 <- v2 / n
+    list(vi = vi, v2 = v2)
 }
 
-isFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), sigma, M = 500, vClose = FALSE, pn = 1, rank = "gehan", omega = FALSE) {
+viClo <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), nres = 500, p0 = 1, p1 = 1, rankweight = "gehan") {
+    p <- ncol(X)
+    clsize <- unlist(lapply(split(id, id), length))
+    n <- length(clsize)
+    N <- sum(clsize)
+    s <- get_si(beta, Y, delta, X, id, weight, nres, rankweight)
+
+    xi <- get_si(beta, Y, delta, X, id, weight, nres, rankweight, rep(delta[unique(id)] == 0, clsize))
+    xiCase <- get_si(beta, Y, delta, X, id, weight, nres, rankweight, rep(delta[unique(id)] == 1, clsize))
+
+    control <- which(delta[unique(id)] == 0)
+    case <- which(delta[unique(id)] == 1)
+
+    v1 <- get_vi(s, id, delta, weight, n, N, 1)$vi / N
+    v21 <- get_vi(xi, id[control], delta[control], weight[control], length(control), N, p0)$v2 / length(control)
+    v22 <- get_vi(xiCase, id[case], delta[case] - 1, weight[case], length(case), N, p1)$v2 / length(case)
+
+    ## v21 <- get_vi(xi, id[control], delta[control], weight[control], length(control), length(control) + length(case), p0)$v2
+    ## v22 <- get_vi(xiCase, id[case], delta[case], weight[case], length(case), length(control) + length(case), p1)$v2
+    prDelta <- table(delta[unique(id)])
+    ## vi <- v1 + v21 * p0 + v22 * p1
+    ## vi <- v1 + v21 * (length(control) / N) + v22 * (length(case) / N)
+    vi <- v1 + v21 * (prDelta[1] / sum(prDelta)) + v22 * (prDelta[2] / sum(prDelta))
+    list(vi = as.matrix(vi), s = s)
+}
+
+isFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), sigma, nres = 500, vClose = FALSE, p0 = 1, p1 = 1, rankweight = "gehan", omega = FALSE) {
   p <- ncol(X)
   clsize <- unlist(lapply(split(id, id), length))
   n <- length(clsize)
   N <- sum(clsize)
   UnMat <- zmat <- ahat <- NULL
   An.inv <- 1
-  if (omega == TRUE && rank == "gehan") {
+  if (omega == TRUE && rankweight == "gehan") {
       vi <- omegaFun(beta, Y, X, delta, clsize, weight) * n## / n ^ 2
   }
-  if (omega != TRUE && vClose == TRUE && rank == "gehan") {
-      vi <- viClo(beta, Y, delta, X, id, weight, M, pn)$vi
+  if (omega != TRUE && vClose == TRUE && rankweight == "gehan") {
+      vi <- viClo(beta, Y, delta, X, id, weight, nres, p0, p1)$vi
   }
-  if (omega != TRUE && vClose == TRUE && rank == "logrank") {
-      vi <- viClo(beta, Y, delta, X, id, weight, M, pn, rank)$vi
+  if (omega != TRUE && vClose == TRUE && rankweight == "logrank") {
+      vi <- viClo(beta, Y, delta, X, id, weight, nres, p0, p1, rankweight)$vi
   }
   if (omega != TRUE && vClose != TRUE) {
-      vi <- viEmp(beta, Y, delta, X, id, weight, M, mb = TRUE, zbeta = FALSE, rank = rank)$vi
+      vi <- viEmp(beta, Y, delta, X, id, weight, nres, mb = TRUE, zbeta = FALSE, rankweight = rankweight)$vi
   }
-  if (rank == "gehan") {
+  if (rankweight == "gehan") {
       An <- abargehanfun(beta, Y, X, delta, clsize, sigma, weight) / n
   }
-  if (rank == "logrank") {
+  if (rankweight == "logrank") {
       An <- abarlogfun(beta, Y, X, delta, clsize, sigma, weight) / n
   }
   if (qr(An)$rank != p) {
@@ -241,7 +265,7 @@ isFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), sigma, M = 50
   list(covmat = covmat, vi = vi, An.msg = An.msg, An.inv = An.inv)
 }
 
-zlFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, vClose = FALSE, pn = 1, rank = "gehan") {
+zlFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), nres = 500, vClose = FALSE, p0 = 1, p1 = 1, rankweight = "gehan") {
   p <- ncol(X)
   clsize <- unlist(lapply(split(id, id), length))
   n <- length(clsize)
@@ -249,14 +273,14 @@ zlFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, vClo
   UnMat <- zmat <- ahat <- unTime <- NULL
   An.inv <- 1
   sigma <- diag(p)
-  UnV <- viEmp(beta, Y, delta, X, id, weight, M, mb = FALSE, zbeta = TRUE, smooth = FALSE, rank = rank)
+  UnV <- viEmp(beta, Y, delta, X, id, weight, nres, mb = FALSE, zbeta = TRUE, smooth = FALSE, rankweight = rankweight)
   zmat <- UnV$zmat
   UnV <- UnV$UnV
   if (vClose == TRUE) {
-    vi <- viClo(beta, Y, delta, X, id, weight, M, pn, rank)$vi
+    vi <- viClo(beta, Y, delta, X, id, weight, nres, p0, p1, rankweight)$vi
   }
   if (vClose != TRUE) {
-    vi <- viEmp(beta, Y, delta, X, id, weight, M, mb = TRUE, zbeta = FALSE, smooth = FALSE, rank = rank)$vi
+    vi <- viEmp(beta, Y, delta, X, id, weight, nres, mb = TRUE, zbeta = FALSE, smooth = FALSE, rankweight = rankweight)$vi
   }
   An <- matrix(0, nrow = p, ncol = p)
   for (i in 1:p) {
@@ -277,7 +301,7 @@ zlFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, vClo
   list(covmat = covmat, vi = vi, An.msg = An.msg, An.inv = An.inv)
 }
 
-huangFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, vClose = TRUE, pn = 1, rank = "gehan") {
+huangFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), nres = 500, vClose = TRUE, p0 = 1, p1 = 1, rankweight = "gehan") {
   p <- ncol(X)
   clsize <- unlist(lapply(split(id, id), length))
   n <- length(clsize)
@@ -286,10 +310,10 @@ huangFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, v
   sigma <- diag(p)
   UnMatV <- NULL
   if (vClose == TRUE) {
-      vi <- viClo(beta, Y, delta, X, id, weight, M, pn, rank)$vi
+      vi <- viClo(beta, Y, delta, X, id, weight, nres, p0, p1, rankweight)$vi
   }
   if (vClose != TRUE) {
-      vi <- viEmp(beta, Y, delta, X, id, weight, M, mb = TRUE, zbeta = FALSE, smooth = FALSE, rank = rank)$vi
+      vi <- viEmp(beta, Y, delta, X, id, weight, nres, mb = TRUE, zbeta = FALSE, smooth = FALSE, rankweight = rankweight)$vi
   }
   qq <- chol(vi)
   ## options(warn = -1)
@@ -304,16 +328,16 @@ huangFun <- function(beta, Y, delta, X, id, weight = rep(1, nrow(X)), M = 500, v
   Z <- rep(1, N)
   newBeta <- matrix(0, ncol = p, nrow = p)
   for ( i in 1:p) {
-      if (rank == "gehan") {
+      if (rankweight == "gehan") {
           bb <- BBsolve(beta, uiFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigma, n = n, Z = Z, weight = weight, smooth = TRUE, constant = qq[,i] * n ^ 0.5, quiet = TRUE)
           newBeta[, i] <- bb$par
       }
-      if (rank == "logrank") {
-          bb <- BBsolve(beta, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigma, n = n, Z = Z, weight = weight, smooth = TRUE, constant = qq[,i] * n ^ 0.5, rank = rank, quiet = TRUE)
+      if (rankweight == "logrank") {
+          bb <- BBsolve(beta, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigma, n = n, Z = Z, weight = weight, smooth = TRUE, constant = qq[,i] * n ^ 0.5, rankweight = rankweight, quiet = TRUE)
           newBeta[, i] <- bb$par
       }
-      if (rank == "PW" | rank == "Prentice-Wilcoxon") {
-          bb <- BBsolve(beta, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigma, n = n, Z = Z, weight = weight, smooth = TRUE, constant = qq[,i] * n ^ 0.5, rank = rank, quiet = TRUE)
+      if (rankweight == "PW" | rankweight == "Prentice-Wilcoxon") {
+          bb <- BBsolve(beta, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = sigma, n = n, Z = Z, weight = weight, smooth = TRUE, constant = qq[,i] * n ^ 0.5, rankweight = rankweight, quiet = TRUE)
           newBeta[, i] <- bb$par
       }
   }
@@ -327,13 +351,13 @@ smoothrr <- function(formula, data, subset, id,
                      Sigma = NULL,
                      weight = NULL,
                      binit = NULL,
-                     M = 100, pn = 1,
+                     nres = 100, p0 = 1, p1 = 1,
                      contrasts = NULL,
                      variance = "ISMB",
-                     rank = "gehan",
+                     rankweight = "gehan",
                      control = aftgee.control()) {
   if (sum(!(variance %in% c("MB", "ZLCF", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB", "js"))) == 8) stop("Invaid variance estimates.")
-  if (!(rank %in% c("gehan", "logrank", "PW", "Prentice-Wilcoxon", "GP"))) stop("Invaid rank weight.")
+  if (!(rankweight %in% c("gehan", "logrank", "PW", "Prentice-Wilcoxon", "GP"))) stop("Invaid rankweight weight.")
   scall <- match.call()
   mnames <- c("", "formula", "data", "weight", "subset", "na.nation", "id")
   cnames <- names(scall)
@@ -351,18 +375,18 @@ smoothrr <- function(formula, data, subset, id,
   if (is.null(weight)) weight <- rep(1, N)
   xnames <- colnames(x) ## [-1]
   if(is.null(Sigma)) {Sigma = diag(ncol(x))} ## x[,-1]
-  out <- smoothrr.fit(Y = log(y[,1]), delta = y[,2], X = as.matrix(x), id = id, weight = weight, binit = binit, Sigma = Sigma, variance = variance, M = M, pn = pn, rank = rank, control = control) ## x[,-1]
+  out <- smoothrr.fit(Y = log(y[,1]), delta = y[,2], X = as.matrix(x), id = id, weight = weight, binit = binit, Sigma = Sigma, variance = variance, nres = nres, p0 = p0, p1 = p1, rankweight = rankweight, control = control) ## x[,-1]
   out$call <- scall
   out$vari.name <- xnames
   class(out) <- "smoothrr"
-  out
+  return(out)
 }
 
 
 smoothrr.fit <- function(Y, delta, X, binit = NULL,
                          Sigma, id, weight = rep(1, nrow(X)),
                          variance = c("MB", "ZLCF", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB", "js"),
-                         M = 100, pn = 1, rank = "gehan",
+                         nres = 100, p0 = 1, p1 = 1, rankweight = "gehan",
                          control = aftgee.control()) {
   p <- ncol(X)
   go <- 1
@@ -382,52 +406,68 @@ smoothrr.fit <- function(Y, delta, X, binit = NULL,
   var.MB <- var.ZLCF <- var.ZLMB <- var.sHCF <- var.sHMB <- var.js <- var.ISCF <- var.ISMB <- NaN
   for (i in 1:control$maxiter) {
       ## Point Estimation
-      if (rank == "gehan") {
+      if (rankweight %in% c("gehan", "PW", "Prentice-Wilcoxon", "GP") & i == 1) {
           tbeta <- system.time(beta.temp2 <- nlm(Ln, p = beta.temp1, other = list(Y, X, delta, clsize, Sigma.temp1, weight, Z), fscale = 0.01)$estimate)
           beta.conv <- 0
       }
-      if (rank == "logrank") {
-          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rank = rank, quiet = TRUE))
+      if (rankweight == "logrank") {
+          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rankweight = rankweight, quiet = TRUE))
           beta.temp2 <- temp$par
           beta.conv <- temp$convergence
       }
-      if (rank == "PW" | rank == "Prentice-Wilcoxon") {
-          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rank = rank, quiet = TRUE))
+      if (rankweight == "PW" | rankweight == "Prentice-Wilcoxon") {
+          beta.temp1 <- beta.temp2
+          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rankweight = rankweight, quiet = TRUE))
           beta.temp2 <- temp$par
-          beta.conv <- temp$convergence
+          print(beta.temp1)
+
+          ## if (abs(beta.temp1 - beta.temp2) < control$abstol) {break}
+          if (max(abs(beta.temp1 - beta.temp2)) >= control$abstol) {
+              beta.temp1 <- beta.temp2
+              next
+          }
+          beta.conv <- ifelse(i == control$maxiter, 1, 0)
       }
-      if (rank == "GP") {
-          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rank = rank, quiet = TRUE))
+      if (rankweight == "GP") {
+          beta.temp1 <- beta.temp2
+          tbeta <- system.time(temp <- BBsolve(beta.temp1, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rankweight = rankweight, quiet = TRUE))
           beta.temp2 <- temp$par
-          beta.conv <- temp$convergence
+          print(beta.temp1)
+
+          ## if (abs(beta.temp1 - beta.temp2) < control$abstol) {break}
+          if (max(abs(beta.temp1 - beta.temp2)) >= control$abstol) {
+              beta.temp1 <- beta.temp2
+              next
+          }
+          beta.conv <- ifelse(i == control$maxiter, 1, 0)
       }
       ## Ends Point Estimation
 
       pass.MB <- pass.ZLCF <- pass.ZLMB <- pass.sHCF <- pass.sHMB <- pass.ISCF <- pass.ISMB <- FALSE
       ZLMB.An.inv <- ZLCF.An.inv <- ISMB.An.inv <- ISCF.An.inv <- js.An.inv <- 1
-      if (M == 0) {break}
+      if (nres == 0) {break}
       ## begin multipler resampling method
       if (sum(variance %in% "MB") > 0 && pass.MB == FALSE) {
-          temp <- matrix(0, nrow = M, ncol = p)
-          if(M == 0) {
+          temp <- matrix(0, nrow = nres, ncol = p)
+          if(nres == 0) {
               var.MB <- NULL
               pass.MB <- TRUE
               if (sum(c("ZLCF", "js", "ZLMB", "sHCF", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
           }
-          for (i in 1:M) {
+          for (i in 1:nres) {
               Z <- rep(rexp(length(clsize)), clsize)
 
-              if (rank == "gehan") {
+              if (rankweight == "gehan") {
                   temp[i,] <- nlm(Ln, p= beta.temp2, other = list(Y, X, delta, clsize, Sigma.temp1, weight, Z), fscale = 0.01)$estimate
               }
-              if (rank == "logrank") {
-                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rank = rank, quiet = TRUE)$par
+              if (rankweight == "logrank") {
+                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rankweight = rankweight, quiet = TRUE)$par
               }
-              if (rank == "PW" | rank == "Prentice-Wilcoxon") {
-                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rank = rank, quiet = TRUE)$par
+              if (rankweight == "PW" | rankweight == "Prentice-Wilcoxon") {
+                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rankweight = rankweight, quiet = TRUE)$par
               }
-              if (rank == "GP") {
-                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rank = rank, quiet = TRUE)$par
+              if (rankweight == "GP") {
+                  temp[i,] <- BBsolve(beta.temp2, uilogFun, Y = Y, X = X, delta = delta, clsize = clsize, sigma = Sigma.temp1, n = n, Z = Z, weight = weight, smooth = TRUE, constant = 0, rankweight = rankweight, quiet = TRUE)$par
               }
           }
           var.MB <- var(temp)
@@ -437,7 +477,7 @@ smoothrr.fit <- function(Y, delta, X, binit = NULL,
 
       ## begin Zeng and Lin's approach, close Si
       if (sum(variance %in% "ZLCF") > 0 && pass.ZLCF == FALSE) {
-          var.ZLCF <- zlFun(beta.temp2, Y, delta, X, id, weight, M, vClose = TRUE, pn, rank)
+          var.ZLCF <- zlFun(beta.temp2, Y, delta, X, id, weight, nres, vClose = TRUE, p0, p1, rankweight)
           ## if (var.ZLCF$An.inv == 0) {print (var.ZLCF$An.msg)}
           ZLCF.An.inv <- var.ZLCF$An.inv
           var.ZLCF <- var.ZLCF$covmat
@@ -447,7 +487,7 @@ smoothrr.fit <- function(Y, delta, X, binit = NULL,
 
       ## begin Zeng and Lin's approach, emprical Si
       if (sum(variance %in% "ZLMB") > 0 && pass.ZLMB == FALSE) {
-          var.ZLMB <- zlFun(beta.temp2, Y, delta, X, id, weight, M, vClose = FALSE, pn, rank)
+          var.ZLMB <- zlFun(beta.temp2, Y, delta, X, id, weight, nres, vClose = FALSE, p0, p1, rankweight)
           ## if(var.ZLMB$An.inv) {print(var.ZLMB$An.msg)}
           ZLMB.An.inv <- var.ZLMB$An.inv
           var.ZLMB <- var.ZLMB$covmat
@@ -457,21 +497,21 @@ smoothrr.fit <- function(Y, delta, X, binit = NULL,
 
     ## begin Huang's approach, close Si
     if (sum(variance %in% "sHCF") > 0 && pass.sHCF == FALSE){
-      var.sHCF <- huangFun(beta.temp2, Y, delta, X, id, weight, M, vClose = TRUE, pn, rank)$covmat
+      var.sHCF <- huangFun(beta.temp2, Y, delta, X, id, weight, nres, vClose = TRUE, p0, p1, rankweight)$covmat
       pass.sHCF <- TRUE
       if (sum(c("js", "sHMB", "ISCF", "ISMB") %in% variance) == 0) {break}
     } ## end Huang's approach, close Si
 
     ## begin Huang's approach, emprical Si
     if (sum(variance %in% "sHMB") > 0 && pass.sHMB == FALSE){
-      var.sHMB <- huangFun(beta.temp2, Y, delta, X, id, weight, M, vClose = FALSE, pn, rank)$covmat
+      var.sHMB <- huangFun(beta.temp2, Y, delta, X, id, weight, nres, vClose = FALSE, p0, p1, rankweight)$covmat
       pass.sHMB <- TRUE
       if (sum(c("js", "ISCF", "ISMB") %in% variance) == 0) {break}
     } ## end Huang's approach, emprical Si
 
     ## begin IS's appraoch, close Si
     if (sum(variance %in% "ISCF") > 0 && pass.ISCF == FALSE) {
-        var.ISCF <- isFun(beta.temp2, Y, delta, X, id, weight, Sigma.temp1, M, vClose = TRUE, pn, rank)
+        var.ISCF <- isFun(beta.temp2, Y, delta, X, id, weight, Sigma.temp1, nres, vClose = TRUE, p0, p1, rankweight)
         ## if(var.ISCF$An.inv == 0){print(var.ISCF$An.msg)}
         ISCF.An.inv <- var.ISCF$An.inv
         var.ISCF <- var.ISCF$covmat
@@ -482,7 +522,7 @@ smoothrr.fit <- function(Y, delta, X, binit = NULL,
 
     ## begin IS's appraoch, emprical Si
     if (sum(variance %in% "ISMB") > 0 && pass.ISMB == FALSE) {
-        var.ISMB <- isFun(beta.temp2, Y, delta, X, id, weight, Sigma.temp1, M, vClose = FALSE, pn, rank)
+        var.ISMB <- isFun(beta.temp2, Y, delta, X, id, weight, Sigma.temp1, nres, vClose = FALSE, p0, p1, rankweight)
         ## if(var.ISMB$An.inv == 0) {print(var.ISMB$An.msg)}
         ISMB.An.inv <- var.ISMB$An.inv
         var.ISMB <- var.ISMB$covmat
@@ -493,7 +533,7 @@ smoothrr.fit <- function(Y, delta, X, binit = NULL,
 
     ## begin JS's iterative approach
     if (sum(variance %in% "js") > 0) {
-      var.js <- isFun(beta.temp2, Y, delta, X, id, weight, Sigma.temp1, omega = TRUE, rank = rank)
+      var.js <- isFun(beta.temp2, Y, delta, X, id, weight, Sigma.temp1, omega = TRUE, rankweight = rankweight)
       ## if(var.js$An.inv == 0) {print(var.js$An.msg)}
       js.An.inv <- var.js$An.inv
       Sigma.temp2 <- var.js$covmat
@@ -511,11 +551,11 @@ smoothrr.fit <- function(Y, delta, X, binit = NULL,
   }
   covmat <- list(MB = var.MB, ZLCF = var.ZLCF, ZLMB = var.ZLMB, sHCF = var.sHCF, sHMB = var.sHMB, ISCF = var.ISCF, ISMB = var.ISMB, js = Sigma.temp2 / n)
   convergence <- if ( i == control$maxiter) 1 else 0
-  out <- list(beta = beta.temp2, covmat = covmat, convergence = convergence, tbeta = tbeta, beta.conv = beta.conv, var.meth = variance,
+  out <- list(beta = beta.temp2, covmat = covmat, convergence = convergence, tbeta = tbeta, beta.conv = beta.conv, beta.conv.at = i, var.meth = variance,
               ZLMB.An.inv = ZLMB.An.inv, ZLCF.An.inv = ZLCF.An.inv, ISMB.An.inv = ISMB.An.inv, ISCF.An.inv = ISCF.An.inv, js.An.inv = js.An.inv)
-  if (ZLMB.An.inv == 0 | ZLCF.An.inv == 0 | ISMB.An.inv == 0 | ISCF.An.inv == 0 | js.An.inv == 0) {
-      print("An is singular")
-  }
+  ## if (ZLMB.An.inv == 0 | ZLCF.An.inv == 0 | ISMB.An.inv == 0 | ISCF.An.inv == 0 | js.An.inv == 0) {
+  ##      print("An is singular")
+  ## }
   class(out) <- "smoothrr.fit"
   return(out)
 }
